unit Extracs2;
{ helpful but non-canonical cs functions }

interface

uses cstypes2,cs2,winprocs,wintypes,sysutils,utils;

Function CardServicesPresent : Boolean;
Function CardPresent(Socket : TSocket) : Boolean; export;
Function CardStatusToBatteryLevel(CardStatus : TEventMask) : TBatteryStatus; export;
Function CheckBattery(Socket : TSocket) : TBatteryStatus; export;
Function SocketCount : TSocket; export;
function IsWriteProtect(Socket : TSocket) : boolean; export;
function CountTuples(Socket : TSocket) : integer;

Function CopyToCard(Handle : TClientHandle; Socket : TSocket;
                         var Buffer; address : longint; size : longint; verify : boolean;
                         Error : PInteger) : Boolean; export;
Function MCopyToCard(Handle : TClientHandle; Socket : TSocket;
                         var Buffer; address : longint; size : longint;
                         verify : boolean; Error : PInteger): Boolean;
Function WCopyToCard(Handle : TClientHandle; Socket : TSocket;
                         var Buffer; address : longint; size : longint;
                         verify : boolean; Error : PInteger): Boolean;

Function CopyFromCard(Handle : TClientHandle; Socket : TSocket;
                         var Buffer; address : longint; size : longint;
                         Error : PInteger) : Boolean; export;
Function MCopyFromCard(Handle : TClientHandle; Socket : TSocket;
                         var Buffer; address : longint; size : longint;
                         Error : PInteger): Boolean;
Function WCopyFromCard(Handle : TClientHandle; Socket : TSocket;
                         var Buffer; address : longint; size : longint;
                         Error : PInteger): Boolean;

{ Low level hack tools }
{ ONLY define for rtMemory resources right now}{SAMFIX}
Function FindResource(Handle : TClientHandle; Resource : TResourceType;
                      Attributes : TResourceAttributes;
                      var base : longint; Physicalwindowsize : longint;
                      Error : PInteger) : boolean; export;

function CSDateToDate(CSDate : word) : TDateTime; export;
function EncodeCSDate(y,m,d : word) : Word; export;
{ When given a WORD coded BCD as XXXX meaning XX.XX returns a string as XX.XX }
function bcd2version(bcd : word) : string; export;

const
  UseMTD : TMTDOptionItems=mtdAuto;
  PrefMTD : TMTDOptionItems=mtdYes;
  MinPhysicalWindowSize : word = 4096;
  PhysicalWindowSize : word = 8192 {(MinPhysicalWindowSize * 2)};

implementation

{$IFDEF DEBUG}
uses main;
{$ENDIF}

Function CardServicesPresent : Boolean;
var
  CS : integer;
  CSInfoBlock : TCSInfo;
begin
  CSInfoBlock.Signature:='XX';
  CS:=CardServices(csGetCardServicesInfo,0,nil,nil,SizeOf(CSInfoBlock),@CSInfoBlock);
  CardServicesPresent:=((CS=0) and (CSInfoBlock.Signature='CS'));
end;

Function CardPresent;
var
  CardStatus : TEventMask;
begin
  Log(Format('CSX-CardPresent Socket=%d',[Socket]));
  If GetStatus(Socket,@CardStatus,nil,nil) then begin
    CardPresent:=CardDetect in CardStatus;
    Log(Format('CSX-CardPresent OK, Result:=%d',[byte(Result)]));
  end else begin
    Log('CSX-CardPresent Failed');
    CardPresent:=False;
  end;
end;

Function CardStatusToBatteryLevel;
begin
  { Mask out battery values }      
  CardStatus:=CardStatus * [BatteryLow,BatteryDead];
  If CardStatus=[BatteryLow] then
    Result:=bsLow
  else if CardStatus=[BatteryDead] then
    Result:=bsDead
  else if CardStatus=[] then
    Result:=bsGood
  else Result:=bsUnknown;
end;

Function CheckBattery;
var
  CardStatus : TEventMask;
begin
  Log(Format('CSX-CheckBattery Socket=%d',[Socket]));
  If GetStatus(Socket,@CardStatus,nil,nil) then begin
    Result:=CardStatusToBatteryLevel(CardStatus);
    Log(Format('CSX-CheckBattery OK, Result:=%d',[byte(Result)]));
  end else begin
    Result:=bsUnknown;
    Log('CSX-CheckBattery Failed');
  end;
end;

Function SocketCount;
var
  Args : TCSInfo;
begin
  If CardServices(csGetCardServicesInfo,0,nil,nil,sizeof(Args),@Args)=0 then
    SocketCount:=Args.SocketCount
  else
    SocketCount:=0;
end;

Function IsWriteProtect;
var
  Error : Integer;
  CardStatus : TEventMask;
begin
  Log(Format('CSX-IsWriteProtect Socket=%d',[Socket]));
  If GetStatus(Socket,@CardStatus,nil,nil) then begin
    IsWriteProtect:=WriteProtect in CardStatus;
    Log(Format('CSX-isWriteProtect OK, Result:=%d',[byte(Result)]));
  end else begin
    IsWriteProtect:=False;
    Log('IsWriteProtectFailed');
  end;
end;

Function CountTuples;
var
  Count : Integer;
begin
  If ValidateCIS(Socket,@Count,nil) then
    CountTuples:=Count
  else
    CountTuples:=-1;
end;

Function CopyToCard;
var
  Code : Integer;
  c : Integer;

  Procedure Copy(MTDOption : TMTDOptionItems);
  begin
    Log(Format('CSX-CopyToCard try method=%d',[word(UseMTD)]));
    Case MTDOption of
      mtdYes: Result:=MCopyToCard(Handle,Socket,Buffer,address,size,verify,@Code);
      mtdNo: Result:=WCopyToCard(Handle,Socket,Buffer,address,size,verify,@Code);
    else
      Code:=ceUndefinedError;
      Result:=False;
    end;
    Log(Format('CSX-CopyToCard result of attempt=%d',[Code]));
    If Assigned(Error) then
      Error^:=Code;
  end;

begin
  if UseMTD<>mtdAuto then begin
    Log('CSX-CopyToCard, non-auto');
    Copy(UseMTD)
  end else begin
    Log('CSX-CopyToCard, auto');
    Copy(PrefMTD);
    If not Result then
      { cyle preferred }
      For c:=Low(MTDAutoOrder) to High(MTDAutoOrder) do begin
        Copy(MTDAutoOrder[c]);
        If Result then begin
          PrefMTD:=MTDAutoOrder[c];
          break;
        end;
      end;
  end;
  Log(Format('CSX-CopyToCard Result=%d',[Code]));
end;

Function MCopyToCard;
var
  MemHandle : TClientHandle;
  Attributes : TWriteMemoryAttributes;
  Code : Integer;
  Attempt : Integer;
  OK : Boolean;
  VerifyBuffer : Pointer;

begin
  Log(Format('MTD CopyToCard Size=%d',[size]));
  MCopyToCard:=False;
  If Verify then
    Attributes:=[wmVerify]
  else
    Attributes:=[];
  Include(Attributes,wmDontEraseFirst);
  if not OpenMemory(Handle,Memhandle,Socket,[],0,Error) then exit;
  Log('Opened OK');
  Attempt:=0;
  OK:=WriteMemory(MemHandle,Attributes,address,Buffer,size,@Code);
  If Assigned(Error)
    then Error^:=Code;
  { Win95 problems.... }
  If not OK and (Code=ceIncompleteWriteRequest) and Win95 and Verify then begin
    Log('Initial write failed, Win95 and verify, trying seperate verify');
    { verify don't work with csMapper if the block is big!!!! }
    Exclude(Attributes,wmVerify);
    If WriteMemory(MemHandle,Attributes,address,Buffer,size,@Code) then begin
      Log('Seperate write OK');
      { write ok... go on to readn & verify, else leave write error message, and OK still false }
      GetMem(VerifyBuffer,Size);
      try
        If ReadMemory(MemHandle,Address,VerifyBuffer^,Size,nil) then begin { nil to preserve old write error }
          Log('Read OK');
          { go on to read, else leave new error and OK still false.... }
          OK:=Compare(Buffer,VerifyBuffer^,Size);
          If OK then
            Code:=0; { else leave as csIncompleteWriteRequest }
          If OK then Log('Verify by hand OK') else Log('Verify by hand failed');
        end;
      finally
        FreeMem(VerifyBuffer,Size);
      end;
    end;
  end;

  If not OK then begin
    Log(Format('Write Failure, %d attempts',[Attempt]));
    CloseMemory(MemHandle,nil); { preserve old error at expense of any newer ones }
    exit;
  end;

  Log(Format('Write OK, %d attempts',[Attempt]));
  if not CloseMemory(MemHandle,Error) then exit;
  Log('Close OK');
  MCopyToCard:=True;
end;

{ Samfix this needn't be so demanding on window size, fix it!! }
Function WCopyToCard;
type
  TByteArray=Array[0..0] of byte;

var
  Resource : TResourceType;
  addressoffset,mapaddress,base,baseoffset,resourcesize : longint;
  ActualWindowSize : longint; { SJL 1997 }
  WHandle : TClientHandle;
  Attributes : TResourceAttributes;
  WindowAttributes : TMemoryWindowAttributes; {SJL 1997}
  MP : Pointer;
  index : word;
  Selector, Xsel : Word;
  VerifyOK : Boolean;

begin
  Log('Window CopyToCard');
  Result:=False;
  if IsWriteProtect(Socket) then begin
    Log('Write protected check failed');
    { fake WP error }
    If Assigned(Error) then Error^:=$1A;
    exit;
  end;

  Resource:=rtMemory;
  Attributes:=[];
  base:=0;
  ResourceSize:=PhysicalWindowSize;

  Log('Look for window resources');
  If not FindResource(Handle,Resource,Attributes,base,resourcesize,Error) then exit;
         { failed - either some failure, or we kept }
         { looking past the end because there were no }
         { suitable resources }

  Log('Requesting window');
  WindowAttributes:=[maWindowEnableControl,ma16bit];
  ActualwindowSize:=PhysicalWindowSize;
  If not RequestWindow(Handle,WHandle,Socket,WindowAttributes,
         base,ActualWindowSize,Error) then exit;

  { having requested the window and GOT it we MUST release it before we quit! }

  { Now MAP window }
  { We must map to the first multiple of MinPhysicalWindowSize <= address }
  Addressoffset:=address mod MinPhysicalWindowSize;
  mapaddress:=address-addressoffset;
  baseoffset:=base+addressoffset;

  Log('Mapping window');
  If not MapMemPage(WHandle,mapaddress,Error) then begin
    ReleaseWindow(WHandle,nil);
    exit;
  end;

  {$IFDEF Windows}
  { now allocate a selector for that ranged }
  Selector:=AllocSelector(DSeg);
  xsel:=SetSelectorBase(Selector,baseoffset);
  xsel:=SetSelectorLimit(Selector,size);
  MP:=ptr(Selector,0);
  {$ELSE}
  MP:=Ptr(BaseOffset,0);
  {$ENDIF}

  Log('Copying to window');
  { Now copy from the card.... }
  move(Buffer,MP^,size);

  Result:=true; { probly is... gets caught if not }
  { now verify copy }
  VerifyOK:=(not Verify) or (Size=0);
  If not VerifyOK then begin
    If not Compare(Buffer,MP^,size) then begin
{    VerifyOK:=True;
    for index:=0 to size-1 do begin
      if byte(MP^)<>TByteArray(buffer)[index] then begin{ There is a verify error }
        Log('Verify error');
        VerifyOK:=False;
{        Break;
      end;}
    end;
  end;

  {$IFDEF WINDOWS}
  FreeSelector(Selector);
  {$ENDIF}

  If not VerifyOK then begin
    ReleaseWindow(WHandle,nil);
    If Assigned(Error) then Error^:=ceIncompleteWriteRequest; {incomplete write request }
    exit;
  end;
  Log('Release window');
  Result:=ReleaseWindow(WHandle,Error);
end;

Function CopyFromCard;
var
  Code : Integer;
  C : Integer;

  Procedure Copy(MTDOption : TMTDOptionItems);
  begin
    Log(Format('CSX-CopyFromCard try method=%d',[word(UseMTD)]));
    Case MTDOption of
      mtdYes: Result:=MCopyFromCard(Handle,Socket,Buffer,address,size,@Code);
      mtdNo: Result:=WCopyFromCard(Handle,Socket,Buffer,address,size,@Code);
    else
      Code:=ceUndefinedError;
      Result:=False;
    end;
    Log(Format('CSX-CopyFromCard result of attempt=%d',[Code]));
    If Assigned(Error) then
      Error^:=Code;
  end;

begin
  if UseMTD<>mtdAuto then begin
    Log('CSX-CopyFromCard, non-auto');
    Copy(UseMTD)
  end else begin
    Log('CSX-CopyFromCard, auto');
    Copy(PrefMTD);
    If not Result then
      { cyle preferred }
      For c:=Low(MTDAutoOrder) to High(MTDAutoOrder) do begin
        Copy(MTDAutoOrder[c]);
        If Result then begin
          PrefMTD:=MTDAutoOrder[c];
          break;
        end;
      end;
  end;
  Log(Format('CSX-CopyFromCard Result=%d',[Code]));
end;

Function MCopyFromCard;
Var
  MemHandle : TClientHandle;
begin
  Log('MTD CopyFromCard');
  MCopyFromCard:=False;
  if not OpenMemory(Handle,Memhandle,Socket,[],0,Error) then exit;
  Log('Opened OK');
  FillChar(Buffer,Size,0);
  Log(Format('CSX-MCopyFromCard: Calling ReadMemory; Size=%d, Address=%d, Buffer=%p',[Size,Address,@Buffer]));
  If not ReadMemory(MemHandle,Address,Buffer,Size,Error) then begin
    Log('Error reading memory');
    CloseMemory(MemHandle,nil); { preserve old error at expense of any newer ones }
    exit;
  end;
  Log('Read OK');
  if not CloseMemory(MemHandle,Error) then exit;
  Log('Closed OK');
  MCopyFromCard:=True;
end;

Function WCopyFromCard;
var
  Resource : TResourceType;
  mapaddress,addressoffset,baseoffset,base,resourcesize : longint;
  ActualWindowSize : longint; { SJL 1997 }
  WHandle : TClientHandle;
  Attributes : TResourceAttributes;
  WindowAttributes : TMemoryWindowAttributes; {SJL 1997}
  MP : Pointer;
  index : word;
  selector, xsel : word;
  answer : longint;

begin
  Log('Window CopyFromCard');
  Result:=False;
  Resource:=rtMemory;
  Attributes:=[];
  base:=0;
  ResourceSize:=PhysicalWindowSize;

  Log('Looking for window resource');
  If not FindResource(Handle,Resource,Attributes,base,resourcesize,Error) then exit;
         { failed - either some failure, or we kept }
         { looking past the end because there were no }
         { suitable resources }

  Log('Requesting window');
  WindowAttributes:=[maWindowEnableControl,ma16bit];
  ActualwindowSize:=physicalWindowSize;
  If not RequestWindow(Handle,WHandle,Socket,WindowAttributes,
         base,ActualWindowSize,Error) then exit;

  { having requested the window and GOT it we MUST release it before we quit! }

  { Now MAP window }
  { We must map to the first multiple of MinPhysicalWindowSize <= address }
  Addressoffset:=address mod MinPhysicalWindowSize;
  mapaddress:=address-addressoffset;
  baseoffset:=base+addressoffset;

  Log('Mapping window');
  If not MapMemPage(WHandle,mapaddress,Error) then begin
    ReleaseWindow(WHandle,nil);
    exit;
  end;

  {$IFDEF Windows}
  { now allocate a selector for that ranged }
  Selector:=AllocSelector(DSeg);
  xsel:=SetSelectorBase(Selector,baseoffset);
  xsel:=SetSelectorLimit(Selector,size);
  MP:=ptr(Selector,0);
  {$ELSE}
  MP:=Ptr(BaseOffset,0);
  {$ENDIF}

  Log('Copying from card');
  { Now copy from the card.... }
  move(MP^,Buffer,size);

  {$IFDEF WINDOWS}
  xsel:=FreeSelector(Selector);
  {$ENDIF}

  Log('Release window');
  Result:=ReleaseWindow(WHandle,Error);
end;

{ See PCMCIA Spec.  The developers guide is very sketchy on this point }
{ The only resource we actually use is a memory resource }
{ This routine could be used to find a memory resource (baseaddress) }
{ which would then be used by "getwindow" }
Function FindResource;
var
  resourcesize : longint;
  CurrentResource : TResourceType;
  CurrentAttributes : TResourceAttributes;
  Code : Integer;

begin
  Log('FindResource');
  ResourceSize:=PhysicalWindowSize;
  CurrentResource:=Resource;
  CurrentAttributes:=Attributes;
  Log('Getting first resource');
  Result:=AdjustResourceInfo(roGetFirstResource,CurrentResource,
          CurrentAttributes,Base,ResourceSize,@Code);
   { Now find a free resource }
  while (result) and
    (((Attributes<>[]) and (Attributes<>[raShared]))or (Resourcesize<physicalWindowSize)) do begin
    { look for a suitable resource }
    CurrentResource:=Resource;
    CurrentAttributes:=Attributes;
    Log('Checking next resource');
    Result:=AdjustResourceInfo(roGetNextResource,CurrentResource,Attributes,Base,ResourceSize,@Code);
  end;
  If Assigned(Error) then
    Error^:=Code;
  Log('ResoucreResult='+IntToStr(Code)+' ('+IntToStr(Byte(Result)));
end;

{ Converts from cardServices special date format to dd/mm/yyyy }
function CSDateToDate(CSDate : word) : TDateTime;
var
  y, m, d : Integer;
begin
  y:=1980+((CSDate shr 9) and $7F);
  m:=(CSDate shr 5) and $f;
  If m=0 then m:=1;
  d:=CSDate and $1f;
  If d=0 then d:=1;
  try
    Result:=EncodeDate(y,m,d);
  except
    result:=y+(m/100)+(d/10000);
  end;
end;

function EncodeCSDate(y,m,d : word) : Word;
begin
  EncodeCSDate:=d+m*32+(y-1980)*512;
end;

{ When given a WORD coded BCD as XXXX meaning XX.XX returns a string as XX.XX }
function bcd2version;
var
  buffer : string[5];
begin
  buffer:=char((bcd and $f) + ord('0'))+Buffer;
  bcd:=bcd shr 4;
  buffer:=char((bcd and $f) + ord('0'))+Buffer;
  bcd:=bcd shr 4;

  buffer:='.'+buffer;

  buffer:=char((bcd and $f) + ord('0'))+Buffer;
  bcd:=bcd shr 4;
  buffer:=char((bcd and $f) + ord('0'))+Buffer;
{REM}
  Result:=Buffer;
end;

end.
