{$C FIXED PRELOAD PERMANENT}
{$F+}
unit Main;

interface

uses windos, pcm, sysutils, wintypes, winprocs, messages, inifiles, classes,
     cstypes2, cs2, extracs2, DPMI, utils, realmode, p16mode, toolhelp, dllini;

const
  Info='Campbell Scientific Combinative Card Services DLL'#31+
        '(C) 1994,1995,1996,1997 Campbell Scientific Ltd'#0;
  DontUseMask : TCSDetected=[];

Procedure DefaultCallback(var Flags, AX, BX, CX, DX, SI, DI, DS, ES, BP : word); export; {far}

Function DLL_Version : longint; export;
Function DLL_Info : PChar; export;
Function Checkversion(Version : Word) : Boolean; export;
Function Status : PChar; export;
Function Style : TCSType; export;
Function AllStyles : TCSDetected; export;
Function MTD : PChar; export;

Procedure log(s : string);

Function NoCardServices(CSFunction : Byte; Handle : TClientHandle; NewHandle : PClientHandle;
                           Buffer : Pointer; Length : word; ArgPointer : Pointer) : integer;

Function CheckInstallation(CSType : TCSType) : TCSType; export;
Procedure SearchInstallation(CSType : TCSType; var Detected : TCSDetected); export;

function Win95WndProc(HWindow: HWnd; Message: Word; WParam: Word; LParam: Longint): Longint; export;
Function CheckCardInsertionComplete : Boolean;

Procedure TimerCallback(Handle : HWND; Msg, IdTimer : word; Time : longint); export;
Function RunSetup : boolean; export;
Function MapCardServices(CSType : TCSType; CSFunction : Byte; Handle : TClientHandle; NewHandle : PClientHandle;
         Buffer : Pointer; Length : word; ArgPointer : Pointer) : integer; export;

Procedure DoNeedRunSetup;
Procedure Block;

type
  EPCCardError=Class(Exception);

var
  Settings : TInifile;

Const
  NeedRunSetup : Boolean=False;
  CSType : TCSType=csUnknown;
  InitialCSType : TCSType=csUnknown;
  KeepLog : Boolean = false;
  Detected : TCSDetected =[];
  win95 : Boolean = true;

implementation

type
  PDelayedEvent=^TDelayedEvent;
  TDelayedEvent=record
    Handle : TClientHandle;
    Event : Word;
    Socket : TSocket;
  end;

const
  InsertionComplete : Boolean=true; { set to false on insertion message }
  Win95Hwnd : HWnd = 0;
  Win95DummyMessage : word = 0;
  MaxInsertionDelay : longint = 1000;

var
  Timer : Word;

Function MapCardServices;
begin
  Case CSType of
    csRealMode: Result:=RealModeCardServices(CSFunction,Handle,NewHandle,Buffer,Length,ArgPointer);
    csProt16:   Result:=Prot16CardServices(CSFunction,Handle,NewHandle,Buffer,Length,ArgPointer);
    csPhoenix:  Result:=PCMCardServices(CSFunction,Handle,NewHandle,Buffer,Length,ArgPointer);
  else
    Result:=NoCardServices(CSFunction,Handle,NewHandle,Buffer,Length,ArgPointer);
  end;
end;

Procedure TimerCallback;
begin
  KillTimer(0,Timer);
  RunSetup;
end;

Function RunSetup;
var
  DLLIniFile : TInifile;
  Procedure SetNone;
  begin
    DLLIniFile:=TIniFile.Create(IniFile);
    DLLIniFile.WriteInteger(Options,CardServicesType,integer(csNone));
    DLLIniFile.Free;
    CSType:=CSNone;
  end;

begin
  Result:=True;
  If WinExec('PCCSetup.exe auto',SW_ShowNormal)<HInstance_Error then begin
    MessageBox(0,'PCCard Setup (PCCSetup.exe) must be run to configure CSL_CS.DLL for PCCard support'+
               #13+'CSL_CS.DLL will configure for no PCCard support',
               'Setup program not found',MB_IconExclamation or MB_OK);
    SetNone;
    Result:=False;
  end;
end;

Procedure AskRunSetup;
begin
  NeedRunSetup:=True;
end;

Procedure DoNeedRunSetup;
begin
  NeedRunSetup:=False;
  RunSetup;
end;

Function IsWin95 : Boolean;
begin
  Result:=(LoWord(GetVersion)=$5F03) or { 3.95 }
          ((Lo(LoWord(GetVersion))=3) and (Hi(LoWord(GetVersion))>=95)) or { 3.>95 }
          (Lo(LoWord(GetVersion))>3); {>3.xx}
end;

Function VTimer : longint;
var
  TimerInfo : TTimerInfo;
begin
  TimerInfo.dwSize:=SizeOf(Timerinfo);
  If not TimerCount(@TimerInfo) then
    Result:=GetTickCount
  else
    result:=TimerInfo.dwmsThisVM;
end;

Procedure Block;
var
  MSG : TMSG;
begin
  { We know this will fail anyway.... but it will yield... }
  PeekMessage(Msg,Win95Hwnd,Win95DummyMessage,Win95DummyMessage,PM_Remove);
end;

function CheckCardInsertionComplete;
var
  CTicks, STicks : longint;
begin
  { loop until timeout... }
  If not InsertionComplete then begin
    CTicks:=VTimer;
    While not InsertionComplete do begin
      Block;
      STicks:=VTimer;
      Dec(STicks,CTicks);
      If STicks>MaxInsertionDelay then
        break;
    end;
  end;
  Result:=InsertionComplete;
end;

Procedure ClearWin95Hacks;
begin
  if Win95Hwnd<>0 then
    DestroyWindow(Win95Hwnd);
end;

Function Win95WndProc;
begin
  Case Message of
    WM_DeviceChange: begin
      If WParam=DBT_DevNodesChanged then
        InsertionComplete:=true;
        result:=0;
    end;
  end;
end;

Procedure PrepareWin95Hacks;
const
  BroadcastWindowClass: TWndClass = (
    style: 0;
    lpfnWndProc: @DefWindowProc;
    cbClsExtra: 0;
    cbWndExtra: 0;
    hInstance: 0;
    hIcon: 0;
    hCursor: 0;
    hbrBackground: 0;
    lpszMenuName: nil;
    lpszClassName: 'TBroadcastWindow');
var
  TempClass: TWndClass;

begin
  Win95:=IsWin95;
  If Win95 then begin
    AddExitProc(ClearWin95Hacks);

    { Allocate window to receive configman broadcasts }
    BroadcastWindowClass.hInstance := HInstance;
    if not GetClassInfo(HInstance, BroadcastWindowClass.lpszClassName, TempClass) then
      WinProcs.RegisterClass(BroadcastWindowClass);
    Win95Hwnd:=CreateWindow(BroadcastWindowClass.lpszClassName, '', 0,
      0, 0, 0, 0, 0, 0, HInstance, nil);
    If Win95HWND=0 then
      Raise Exception.Create('Cannot allocate broadcast windows handle');
    SetWindowLong(Win95Hwnd, GWL_WNDPROC, longint(@Win95WndProc));
    Win95DummyMessage:=RegisterWindowMessage('CSL-CS-Configman-Hack');
  end;
end;

Procedure DefaultCallback;
{ It will deal with info requests, and then pass on to the right handler }
{var
  PrivateCallBack : CallBack_Procedure;}

begin
  If Win95 and (csType=csRealMode) then
    Case AX of
      cbCardInsertion: InsertionComplete:=False;
      { was no doubt set false before, dur to artificial insertion, yet we have no }
      { broadcast pending, so fix now. It doesn;t matter that that card isn't in }
      { as long as we don't access the card when it IS in BEFORE the broadcast }
      cbRegistrationComplete: InsertionComplete:=True;
    end;

  PostMessage(DI,WM_Command,IDM_Callback,(longint(ax) shl 16) or longint(cx));

  AX:=0;		{ Return Code for card services }
  flags:=flags and not fcarry;	{ clear carry means OK }
  { although we havent specifically tested for certain MUST-RESPOND }
  { callback functions, such as RESET_REQUEST etc, we return OK to ALL }
  { functions, as we do not object if some other client WISHES to do this }
end;

Function NoCardServices;
begin
  Result:=ceFunctionNotSupported; { Function not supported }
end;

{ A Cheating function, turns numeric date into a word coded in the required format }
function makedate(day,month,year : integer) : word;
begin
  makedate:=day+month*32+(year-1980)*512;
end;

Function Style;
begin
  Result:=CSType;
end;

Function AllStyles;
begin
  Result:=Detected;
end;

Function MTD;
begin
  If UseMTD=mtdAuto then
    Result:=AutoMTDTypes[PrefMTD]
  else
    Result:=MTDTypes[UseMTD];
end;

Function Status;
begin
  Result:=nil;
  If CallbackSelectorUsedTwice then Result:='WARNING: Callback Selector used twice.';
end;


Procedure log;
const
  c : word=0;
begin
  if keeplog then
    begin
      Settings:=TIniFile.Create(IniFile);
      Settings.WriteString('Log',inttostr(c),s);
      inc(c);
      Settings.Free;
    end;
end;

function CheckVersion;
begin
  Result:=Version=DLL_Version;
end;

Function DLL_Version;
begin
  result:=$1300;
end;

Function DLL_Info;
begin
  Result:=Info;
end;

Procedure SearchInstallation;
begin
  { First check according to preference from ini file }
  If CSType in [{csUnknown,}csAuto] then begin
    Detected:=[];
    Detected:=Detected+CheckRealMode;
    Detected:=Detected+Check16bitProtectedMode;
    Detected:=Detected+CheckPhoenixDLL;
    Detected:=Detected;
  end else
    Detected:=[CSType];
end;

Function CheckInstallation;
const
  KeepLooking : TCSDetected=[csUnknown,csAuto];
begin
  SearchInstallation(CSType,Detected);
  { Now set type based on Detected }
  If (cstype=csProt16) or ((csType in KeepLooking) and (csProt16 in Detected) and not (csProt16 in DontUseMask)) then begin
    csType:=csProt16;
{    If UseMTD=mtdAuto then UseMTD:=mtdNo;}{SYSTEMSOFT Can now}
    @CardServices:=@Prot16CardServices;
  end;
  If (csType=csRealMode) or
     ((csType in KeepLooking) and (csRealMode in Detected) and not (csRealMode in DontUseMask)) then begin
    csType:=csRealMode;
    @CardServices:=@RealModeCardServices;
  end;
  { Most unlikely, we would probably get real mode instead.... }
  If (csType=csPhoenix) or ((csType in KeepLooking) and (csPhoenix in Detected) and not (csPhoenix in DontUseMask)) then begin
    InstallPCM;
    csType:=csPhoenix;
    @CardServices:=@PCMCardServices;
  end;
  If (csType=csNone) or (csType in KeepLooking)  then begin
    csType:=csNone;
    @CardServices:=@NoCardServices;
  end;
  Result:=CSType;
end;

begin
  @CardServices:=@NoCardServices;
{  SetupDelayedEvents;}
  { Test for win95}
  PrepareWin95Hacks;

  Settings:=TIniFile.Create(IniFile);
  Settings.EraseSection('LOG');
  KeepLog:=Settings.ReadBool(Options,'KeepLog',False);
  UseMTD:=TMTDOptionItems(Settings.ReadInteger(Options,'UseMTD',0));
  Integer(DontUseMask):=Settings.ReadInteger(Options,'Ignore',0);
  MaxInsertionDelay:=Settings.ReadInteger(Options,'MaxInsertionDelay',3000);
  InitialCSType:=TCSType(Settings.ReadInteger(Options,CardServicesType,integer(csUnknown)));
  CSType:=InitialCSType;
  Settings.Free;
  Log('Read Settings');
end.


