unit P16mode;

interface

uses cstypes2, cs2, sysutils, wintypes, windos;

Function Prot16CardServices(CSFunction : Byte; Handle : TClientHandle; NewHandle : PClientHandle;
                           Buffer : Pointer; Length : Word; ArgPointer : Pointer) : integer;
Function Check16bitProtectedMode : TCSDetected;

implementation

uses main;

Function Prot16CardServices;
var
  CarryFlag : boolean;
  t : pointer;
  Reg : TRegisters;
  tw : word;

begin
  Reg.AH:=$AF;
  Reg.AL:=byte(CSFunction);
  Reg.DX:=Handle;
  Reg.CX:=Length;
  Reg.DI:=seg(Buffer^);
  Reg.SI:=Ofs(Buffer^);
  Reg.ES:=Seg(ArgPointer^);
  Reg.BX:=Ofs(ArgPointer^);
  { sanity }
  Reg.DS:=DSeg;

  Intr($1A,Reg);              { Call Card Services }

  If Assigned(NewHandle) then
    NewHandle^:=Reg.DX;

  CarryFlag:=((Reg.flags AND fCarry)=fCarry);
  Result:=Reg.AX;
end;

Function Check16bitProtectedMode : TCSDetected;
var
  CSInfo : TCSInfo;

  Function CardServicesPresent : TCSDetected;
  var
    CS : integer;
    CSProt : Boolean;
    GFC : Array [0..1] of word;
  begin
    Result:=[];
    { be very careful here, in case we see a real-mode mirror }
    { do the odd test first.... }
    { First do GetFirstClient.  This has a read-only arg block }
    { If it reads in real mode it will be junk, and hopefully fail... }
    { If it does fail, its either real-mode, or csmapper (which doesn't support it) }
    { and so therefore also real-mode (csmapper is realmode only ) }
    { If it succeeds we do more detailed tests }
    Log('-Doing GetFirstClient trick test');
    GFC[0]:=0;
    GFC[1]:=0;
    CS:=Prot16CardServices(csGetFirstClient,0,nil,nil,SizeOf(GFC),@GFC);
    If CS=ceFunctionNotSupported then begin
      Log('-Failed ceFunctionNotSupported - probably csMapper - failed');
      { csmapper - this one is easy }
      Result:=[csRealProt16];
      exit;
    end;
    If (CS<>ceNoMoreItemsAvailable) and (CS<>0) then begin
      Log(Format('-Failed %d - lucky break, probably real mode - failed',[CS]));
      { only valid error *we feel* for prot mode, so must be real mode }
      Result:=[csRealProt16];
      exit;
    end;
    { seemed to succeed... this could be a fluke... }
    { we'll try and MAKE it fail and see if it still does.... }
    GFC[0]:=$FFFF;
    GFC[1]:=1; { try socket $FFFF }
    CS:=Prot16CardServices(csGetFirstClient,0,nil,nil,SizeOf(GFC),@GFC);
    If CS=0 then begin
      log('-Failed - returned OK on failure spoof... - failed sadly');
      { ahahh - should have failed... }
      Result:=[csRealProt16];
      exit;
    end;

    CSInfo.Signature:='XX';
    CS:=Prot16CardServices($0B,0,nil,nil,SizeOf(CSInfo),@CSInfo);
    If (CS=0) then begin
      CSProt:=(CSInfo.Signature='CS');
      If CSProt then
        Result:=Result+[csProt16]
      else { bad because we couldn't detect it before }
        Result:=Result+[csBadRealProt16]
    end;
  end;

begin
  Log('Checking for 16 bit protected mode support');
  Result:=[];
  Result:=CardServicesPresent;
end;

end.

{ This is not simple }
{ The PM intr could be mirrored to real mode but without selector mapping }
{ and we don't want our selector to cause real mode memory to get overwritten }
(*Function Check16bitProtectedMode : TCSDetected;
var
  RealReal : integer;
  ProtReal : Integer; { protected mode selector to real ram }
  DoublePointer : Pointer;
  RealPointer : Pointer;
  CSInfo : ^TCSInfo;
  CSInfohandle : THandle;
  CSProt, CSReal : Boolean;

  { Special version to use our matched selector buffer }
  Function CardServicesPresent : TCSDetected;
  var
    CS : integer;
  begin
    Result:=[];
    CSInfo^.Signature:='XX';
    CS:=Prot16CardServices($0B,0,nil,nil,SizeOf(CSInfo),DoublePointer);
    If (CS=0) then begin
      CSProt:=(CSInfo^.Signature='CS');
      CSReal:=(TCSInfo(RealPointer^).Signature='CS');
      If CSProt then
        Result:=Result+[csProt16];
      If CSReal then
        Result:=Result+[csRealProt16];
    end else
      Result:=[csNone];
  end;

begin
  Result:=[];
  CSInfo:=nil;
  CSInfoHandle:=0;

  try
    CSInfoHandle:=GlobalAlloc(GMEM_Fixed or GMEM_DDEShare,SizeOf(TCSInfo));
    GlobalFix(CSInfoHandle);
    CSInfo:=GlobalLock(CSInfoHandle);
    If not GetSelectorSegmentMatchingPair(RealReal,ProtReal,SizeOf(CSInfo^),CSInfo) then
      exit;
    { RealReal is also a form of ProtProt }
    { Double Pointer points to PROT buffer in PROT mode }
    { and points to REAL buffer in REAL mode }
    try
      DoublePointer:=Ptr(RealReal,0);
      { Do a sanity check, do DoublePointer and @CSInfo really access the same RAM? }
      CSInfo^.InfoLen:=0;
      If TCSInfo(DoublePointer^).InfoLen<>0 then
        Raise Exception.Create('Double buffer allocation in error');
      CSInfo^.InfoLen:=1;
      If TCSInfo(DoublePointer^).InfoLen<>255 then
        Raise Exception.Create('Double buffer allocation in error');
      { Real Pointer points to REAL buffer in PROT mode, i.e. now }
      RealPointer:=Ptr(ProtReal,0);
      Result:=CardServicesPresent;
    finally
      FreeSelectorSegmentMatchingPair(RealReal,ProtReal);
    end;
  finally
    If (CSInfo<>nil) and (CSInfoHandle<>0) then
      GlobalUnlock(CSInfohandle);
    If CSInfoHandle<>0 then begin
      GlobalUnfix(CSInfoHandle);
      GlobalFree(CSInfoHandle);
    end;
  end;
end;
*)

