{$C FIXED PRELOAD PERMANENT}
unit Pcm;

interface

uses wintypes, winprocs, sysutils, cstypes2;

type
  TPCMCardServices = Function(Func : Word; Handle : Pointer;
                      Point : Pointer;
                      ArgLength : Word; ArgPointer : Pointer) : Word;
  TPCMRegister = Function(WHandle : HWND) : Boolean;
  TPCMDeregister = Function(WHandle : HWND) : Boolean;
  TPCMGetHandle = Function : Word;

const
  PCM_CS_ModuleHandle : THandle = 0;
  PCM_CS_MODULE='pcmwind.dll'#0;
  IDM_Callback=805; { Defined for Phoenix Card Sevices }

var
  _PCMCardServices : TPCMCardServices;
  _PCMRegister : TPCMRegister;
  _PCMDeregister : TPCMDeregister;
  _PCMGetHandle : TPCMGetHandle;

Function CheckPhoenixDLL : TCSDetected;
Function InstallPcm : Boolean;
Procedure DeInstallPCM;

{ PCMCard services does NOT support a callback, but only windows messages }
Function PCMCardServices(CSFunction : Byte; Handle : TClientHandle; NewHandle : PClientHandle;
                           Buffer : Pointer; Length : Word; ArgPointer : Pointer) : integer;

implementation

uses main;

var
  OldExitProc : Pointer;

{ This is simple enough }
Function CheckPhoenixDLL : TCSDetected;
var
  CS : integer;
  CSInfoBlock : TCSInfo;
begin
  Log('Checking for Phoenix support');
  Result:=[];

  If not InstallPCM then exit;

  CSInfoBlock.Signature:='XX';
  CS:=PCMCardServices(csGetCardServicesInfo,0,nil,nil,SizeOf(CSInfoBlock),@CSInfoBlock);
  If ((CS=0) and (CSInfoBlock.Signature='CS')) then begin
    Result:=[csPhoenix];
    Log('Found Phoenix support');
  end;
  { I think we will unload here... }
  DeInstallPCM;
end;

Procedure DeInstallPCM;
begin
  log('De-installing '+PCM_CS_MODULE);
  If PCM_CS_ModuleHandle>HInstance_Error then begin
    FreeLibrary(PCM_CS_ModuleHandle);
    Log('Unloading');
  end else
    Log('Was not loaded');
  PCM_CS_ModuleHandle:=0;
  @_PCMCardServices:=nil;
  @_PCMRegister:=nil;
  @_PCMDeregister:=nil;
  @_PCMGetHandle:=nil;
end;

procedure MyExit; far;
begin
  Log('PCM Exitproc');
  ExitProc:=OldExitProc;
  DeInstallPCM;
end;

Function InstallPcm;
var
  TErrorMode : Word;
begin
  Log('Loading '+PCM_CS_MODULE);
  Result:=False;

  try
    TErrorMode:=SetErrorMode(SEM_NOOPENFILEERRORBOX);
    try
      PCM_CS_ModuleHandle:=LoadLibrary(PCM_CS_MODULE);
    finally
      SetErrorMode(TErrorMode);
    end;
  except
    Log('Exception while loading '+PCM_CS_MODULE);
    PCM_CS_ModuleHandle:=0;
  end;

  If PCM_CS_ModuleHandle>=HInstance_Error then begin
    Result:=True;
    log('Loaded '+PCM_CS_MODULE+' OK');
    @_PCMCardServices:=GetProcAddress(PCM_CS_ModuleHandle,ptr(0,4));
    If @_PCMCardServices=nil then Result:=False;
    @_PCMRegister:=GetProcAddress(PCM_CS_ModuleHandle,ptr(0,2));
    If @_PCMRegister=nil then Result:=False;
    @_PCMDeregister:=GetProcAddress(PCM_CS_ModuleHandle,ptr(0,3));
    If @_PCMDeregister=nil then Result:=False;
    @_PCMGetHandle:=GetProcAddress(PCM_CS_ModuleHandle,ptr(0,7));
    If @_PCMGetHandle=nil then Result:=False;
    If Result=False then begin
      Log('Error setting PCM procedural addressed');
      DeInstallPCM;
    end else begin
      Log('PCM installed OK');
    end;
  end;
end;

Function PCMCardServices;
var
  PCMHandle : Word;
  PCMResult : Integer;
  BufData : Longint;
  BufProtReal, BufRealReal : Pointer;
  Size : longint;
  Window : HWND;

begin
  Log('PCM-CS Function='+IntToStr(csFunction)+' handle='+IntToStr(Handle));
  try
    { preparation of secondary buffers }
    If CSFunction in [csReadMemory,csWriteMemory] then begin
      { TReadMemoryArgs and TWriteMemoryArgs are very similar }
      Size:=TReadMemoryArgs(ArgPointer^).Count;
      BufData:=GlobalDosAlloc(Size);
      BufProtReal:=Ptr(LoWord(BufData),0);
      BufRealReal:=Ptr(HiWord(BufData),0);
    end else begin
      BufRealReal:=Buffer;
      BufProtReal:=nil;
    end;

    { before the fact mapping }
    Case CSFunction of
      csWriteMemory: Move(Buffer^,BufProtReal^,Size);
    end;

    { Are they registering or deregistering? }
    Case csFunction of
      csRegisterClient: begin
        { we presume windows handle is passed in clientdatabindings.DI }
        { this is a hack, but matches what default RegisterClient does }
        Window:=TRegisterClientBlock(ArgPointer^).ClientDataBindings.DI;
        Log(Format('PCM_RegisterClient intercepted, Window=%d',[window]));
        If _PCMRegister(Window) then begin
          PCMResult:=0;
          PCMHandle:=_PCMGetHandle;
        end else
          PCMResult:=ceUndefinedError; { We don't really know what error... }
      end;
      csDeregisterClient: If _PCMDeregister(Handle) then
          PCMResult:=0
        else
          PCMResult:=ceUndefinedError;
    else
      { PCM DLL copies the argbuffer for us, but not the other buffer }
      PCMHandle:=Handle;
      { Note how we use BufRealReal and not Buffer }
      PCMResult:=_PCMCardServices(CSFunction,@PCMHandle,BufRealReal,Length,ArgPointer);
    end;

    Result:=PCMResult;
    If NewHandle<>nil then
      NewHandle^:=PCMHandle;

    { After-the-fact mapping }
    Case CSFunction of
      csReadMemory: Move(BufProtReal^,Buffer^,Size);
    end;

  finally
    If BufProtReal<>nil then
      GlobalDosFree(Seg(BufProtReal^))
  end;
  Log('PCM-CS Result='+IntToStr(Result)+' NewHandle='+IntToStr(PCMHandle));
end;

begin
  OldExitProc:=ExitProc;
  ExitProc:=@MyExit;
end.
