unit Realmode;

interface

uses sysutils, cs2, cstypes2, dpmi, winprocs, wintypes, classes, callback;

Function CheckRealMode : TCSDetected;
Function RealModeCardServices(CSFunction : Byte; Handle : TClientHandle; NewHandle : PClientHandle;
                           Buffer : Pointer; Length : Word; ArgPointer : Pointer) : integer;

const
  CallbackSelectorFree : Boolean=True; { a lock on the selector }
  CallbackSelectorUsedTwice : Boolean=False; { more for educational purposed, to }
                                      { detect if we use it twice at once... }


implementation

uses main;

const
  CallbackRecords : TList=nil;
  CallbackDPMIReg : TDPMI_Reg=();
  CallbackSelector : Word=0;       { used to map to real mode callback ES }

type
  TCallbackRecord=Record
    Handle : longint;
    Callback : Pointer;
  end;


Procedure LogRealModeCallback(Handle : longint; Callback : Pointer);
var
  CallbackRecord : ^TCallbackRecord;
begin
  Log(Format('Logging realmode callback %p on handle %d',[Callback,handle]));
  New(CallbackRecord);
  CallbackRecord^.Handle:=Handle;
  CallbackRecord^.Callback:=Callback;
  CallbackRecords.Add(CallbackRecord);
end;

Procedure ReleaseLoggedRealModeCallback(Handle : longint);
var
  c : Integer;
  CallbackRecord : ^TCallbackRecord;
begin
  Log(Format('Releasing all callbacks on handle %d',[handle]));
  c:=0; while c<CallbackRecords.Count do begin
    CallbackRecord:=CallbackRecords[c];
    If CallbackRecord^.Handle=Handle then begin
      CallbackRecords.Delete(c);
      ReleaseRealModeCallback(CallbackRecord^.Callback);
      Dispose(CallbackRecord);
      break;
    end;
    inc(c);
  end;
end;

{$S-}
Procedure DPMICallbackController(Flags, rAX, rBX, rCX, rDX, rSI, rDI, rDS, rES, rBP: Word); Interrupt;
var
  DPMIReg : ^TDPMI_Reg;
  ClientRecord : PClientRecord;
begin { These registers are NOT the callback registers.... we still need to }
      { obtain then from ES:DI as TDPMI_Reg }
  DPMIReg:=Ptr(rES,rDI);

  { DPMIReg represents the callback register state }

  { First fill in call back values with what they should be... }
  With DPMIReg^ do begin
    { These DI,SI are actually longint (DPMI made it) }
    ClientRecord:=Ptr(word(DI),word(SI));
    DI:=ClientRecord^.ClientdataBindings.DI;
    DS:=ClientRecord^.ClientdataBindings.DS;
    SI:=ClientRecord^.ClientdataBindings.SI;
  end;

  If CallbackSelectorFree then begin

    { There is not try-finally here, cos if we crash, we are done for anyway... }
    CallbackSelectorFree:=False;
    SetSelectorBase(CallbackSelector,longint(DPMIReg^.ES)*longint(16));
    SetSelectorLimit(CallbackSelector,$10000); { 64K }

    if (byte(DPMIReg^.AX)=cbClientInfo) then begin      { Was it a request to supply client info? }
      CopyClientInfo(ClientRecord^.ClientInfo,
                     TClientInfo(Ptr(CallbackSelector,DPMIReg^.BX)^));
    end else
      { Now make callback }
      With DPMIReg^ do
        ClientRecord^.Callback(Flags,Word(AX),Word(BX),Word(CX),Word(DX),
                               word(SI),Word(DI),DS,CallbackSelector,Word(BP));

    SetSelectorLimit(CallbackSelector,0);
    SetSelectorBase(CallbackSelector,0);
    CallbackSelectorFree:=True;
  end else
    CallbackSelectorUsedTwice:=True;

  { Now fake return as per the DPMI spec }
  {DS:SI is prot mode version of SS:SP}
  DPMIReg^.IP:=PWord(ptr(rDS,rSI))^;
  DPMIReg^.CS:=PWord(ptr(RDS,RSI+2))^;
  { And fake the popping on the realmode IP, add 4 }
  Inc(DPMIReg^.SP,4);
end;

{ This is simple enough }
Function CheckRealMode : TCSDetected;
var
  CS : integer;
  CSInfoBlock : TCSInfo;
begin
  Log('Checking for RealMode support');
  Result:=[];
  CSInfoBlock.Signature:='XX';
  CS:=RealModeCardServices(csGetCardServicesInfo,0,nil,nil,SizeOf(CSInfoBlock),@CSInfoBlock);
  If ((CS=0) and (CSInfoBlock.Signature='CS')) then begin
    Log('Found RealMode support');
    Result:=[csRealMode];
  end;
end;

{ He we intercept and do all the magic, as our client is protected mode }
{ we always copy the ArgBuffer back and forth to a realmode buffer, and }
{ if neccessary we decode the arg buffer to know how much buffer to copy }
{ back and forth }
{ for registration and deregistration we handle allocation of real mode }
{ callbacks too.... }
Function RealModeCardServices;
var
  ArgData : Longint;
  ArgProtReal, ArgRealReal : Pointer;
  BufData : Longint;
  BufProtReal, BufRealReal : Pointer;
  Size : Word;
  RMCallback : Pointer;

  DPMIReg : TDPMI_Reg;
  DPMIRegPtr : Pointer;
  DPMIResult : Boolean;
begin
  { Here is a win95 hack.... }
  { If they are trying to open-memory, block until insertion complete }
  If csFunction=csOpenMemory then
    If not CheckCardInsertionComplete then begin
      Result:=ceNoCardPresent; { best attempt at meaningful error }
      Exit;
    end;

  Log(Format('RealMode-CS Function=%d, Handle=%d',[csFunction,Handle]));
  ArgData:=0;
  BufProtReal:=nil;
  BufRealReal:=nil;
  try
    { preparation of secondary buffers }
    If (CSFunction=csReadMemory) or (CSFunction=csWriteMemory) then begin
      { TReadMemoryArgs and TWriteMemoryArgs are very similar }
      Size:=TReadMemoryArgs(ArgPointer^).Count;
      BufData:=GlobalDosAlloc(Size);
      If BufData=0 then begin
        Log(Format('Cannot allocate DOS buffer size %d',[Size]));
        Result:=-ceCannotAllocateBufferBuffer;
        Exit;
      end;
      BufProtReal:=Ptr(LoWord(BufData),0);
      BufRealReal:=Ptr(HiWord(BufData),0);
    end else begin
      BufRealReal:=Buffer;
      BufProtReal:=nil;
    end;

    { before the fact mapping }
    Case CSFunction of
      csRegisterClient: begin
        { Fix callback routine!! SAMFIX what about the user supplied callback? }
        { Note how we use BufRealReal }
        BufRealReal:=GetRealModeCallback(@DPMICallbackController,CallbackDPMIReg);
      end;
      csWriteMemory: Move(Buffer^,BufProtReal^,Size);
    end;

    { We always want to map arguments to real mode }
    ArgData:=GlobalDosAlloc(Length);
    try
      ArgProtReal:=Ptr(LoWord(ArgData),0);
      ArgRealReal:=Ptr(HiWord(ArgData),0);
      { Copy arguments }
      Move(ArgPointer^,ArgProtReal^,Length);

      { zero for safety }
      FillChar(DPMIReg,SizeOf(DPMIReg),0);
      DPMIRegPtr:=@DPMIReg;
      With DPMIReg do begin
        AX:=($AF shl 8) or byte(CSFunction);
        DX:=Handle;
        CX:=Length;
        { Note hgow we use BufRealReal and not Buffer }
        DI:=HiWord(longint(BufRealReal));
        SI:=LoWord(longint(BufRealReal));
        ES:=HiWord(longint(ArgRealReal));
        BX:=LoWord(longint(ArgRealReal));
        DS:=DSeg;
      end;

      asm
        mov bl,$1a
        mov bh,0
        mov cx,0
        mov es,DPMIRegPtr.longrec.hi
        mov di,DPMIRegPtr.longrec.lo
        mov ax,$0300
        int $31
        mov DPMIResult,1
        jnc @@OK
        mov DPMIResult,0
        @@OK:
      end;
      Move(ArgProtReal^,ArgPointer^,Length);
    finally
      GlobalDosFree(Seg(ArgProtReal^))
    end;

    Result:=DPMIReg.AX;
    If NewHandle<>nil then
      NewHandle^:=DPMIReg.DX;
    If not DPMIResult then
      Result:=-1;

    { After-the-fact mapping }
    Case CSFunction of
      csReadMemory: Move(BufProtReal^,Buffer^,Size);
      csRegisterClient: begin
        If Result<>0 then
          ReleaseRealModeCallback(Buffer)
        else
          LogRealModeCallBack(NewHandle^,Buffer);
      end;
      csDeRegisterClient: If Result=0 then ReleaseLoggedRealModeCallback(Handle);
    end;

  finally
    If BufProtReal<>nil then
      GlobalDosFree(Seg(BufProtReal^))
  end;
  Log(Format('RealMode-CS Result=%d, NewHandle=%d',[Result,DPMIReg.DX]));
end;

Procedure Exiting; far;
begin
  CallBackRecords.Free;
  If CallbackSelector<>0 then FreeSelector(CallbackSelector);
end;

begin
{  CallbackSelector:=CreateAliasDescriptor(DSeg);}
  CallbackSelector:=AllocSelector(DSeg);
  If CallbackSelector=0 then
    Raise Exception.Create('Cannot allocate callback selector');
  SetSelectorLimit(CallbackSelector,0);
  SetSelectorBase(CallbackSelector,0);

  CallbackRecords:=TList.Create;
  AddExitProc(Exiting);
  FillChar(CallbackDPMIReg,SizeOf(CallbackDPMIReg),0);
end.

(*

  TRealModeStack=class(TObject)
  protected
    FUSed : Boolean;
    FSelector : Integer;
    FParagraph : integer;
    FSize : Integer;
    FBottom : pointer;        { real mode pointer }
    FTop : Pointer;           { real mode pointer }
    FPTop : POinter;
    FPBottom : Pointer;
  public
    Property Size : Integer Read FSize;
    Property Bottom : Pointer Read FBottom;
    Property Top : Pointer Read FTop;
    Property PTop : Pointer read FPTop;
    Property PBottom : Pointer Read FPBottom;
    Property USed : Boolean Read FUsed Write FUsed;
    Constructor Create(Size : Integer);
    Destructor Destroy; override;
  end;

Constructor TRealModeStack.Create;
var
  Selectors : longint;
begin
  Inherited Create;
  Selectors:=GlobalDosAlloc(Size);
  If Selectors=0 then
    Raise Exception.Create('Cannot allocate DOS stack size '+IntToStr(Size));
  FSelector:=HiWord(Selectors);
  FParagraph:=LoWord(Selectors);
  FSize:=Size;
  FBottom:=Ptr(FSelector,0);
  FPBottom:=Ptr(FParagraph,0);
  FTop:=Ptr(FSelector,Size);
  FPTop:=Ptr(FParagraph,Size);
end;

Destructor TRealModeStack.Destroy;
begin
  GlobalDosFree(FParagraph);
  Inherited Destroy;
end;


Function RealModeRegisterClient;
var
   RegisterClientBlock : TRegisterClientBlock; { Used for client registration info }
   RMCallback : Pointer;
   rES, rDI : Integer;
   Code : Integer;

begin
  Result:=False;
  With RegisterClientBlock do begin
    RMCallback:=GetRealModeCallback(@DPMICallbackController);

    Attributes:=U_Attributes;
    EventMask:=U_Events;
    ClientDataBindings.DS:=DSeg;       { cos we need our DS value }
    ClientDataBindings.SI:=0;
    ClientDataBindings.DI:=Callback;   { so we can post a message }
    ClientDataBindings.R:=0;           { reserved }
    Version:=U_Version;
  end;

  Code:=CardServices($10,0,@NewHandle,RMCallback,SizeOf(RegisterClientBlock),@RegisterClientBlock);
  If Assigned(Error) then Error^:=Code;
  Result:=Code=0;
  If Code<>0 then
    ReleaseRealModeCallback(RMCallback)
  else
    LogRealModeCallBack(NewHandle,RMCallback);
end;

Function RealModeDeRegisterClient;
var
  CS : integer;
begin
  Result:=False;
  CS:=CardServices($02,Handle,nil,nil,0,nil);
  Result:=CS=0;
  If CS=0 then ReleaseLoggedRealModeCallback(Handle);
end;

(*Function RealModeDeRegisterClient(Handle : TClientHandle) : Boolean; export;
Function RealModeRegisterClient(var NewHandle : TClienthandle;
                        U_Attributes : TClientAttributes;
                        U_Events : TEventMask;
                        U_Version: word;
                        U_ClientName,
                        U_VendorName: string;
                        CallBack : HWND;
                        Error : PInteger) : Boolean;

*)
