unit Clients;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Menus,
  Dialogs, StdCtrls, Buttons, ExtCtrls, cardserv, Tabs, csTypes2, sysutils,
  utils, TabNotBk, modify, Inifiles, DLLIni;

type
  TSetup = class(TForm)
    StatusBar: TPanel;
    TabbedNotebook1: TTabbedNotebook;
    GroupBox1: TGroupBox;
    ClientName: TLabel;
    Revision: TLabel;
    RevisionDate: TLabel;
    Target: TLabel;
    Header: THeader;
    ClientTabs: TTabSet;
    Header1: THeader;
    CardServices: TLabel;
    CSVersion: TLabel;
    CSRevision: TLabel;
    CSSockets: TLabel;
    Panel1: TPanel;
    IOD: TRadioButton;
    MTD: TRadioButton;
    MCD: TRadioButton;
    Insertion: TCheckBox;
    ExInsertion: TCheckBox;
    VendorText: TMemo;
    Label2: TLabel;
    Label1: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    BitBtn1: TBitBtn;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    Label6: TLabel;
    Label7: TLabel;
    WindowsVersion: TLabel;
    DOSVersion: TLabel;
    IsWindows95: TLabel;
    Label9: TLabel;
    Panel2: TPanel;
    CSMapperGroup: TGroupBox;
    CSMapperInfo: TMemo;
    VMMapperGroup: TGroupBox;
    VMMapperInfo: TMemo;
    Label10: TLabel;
    StateTabs: TTabSet;
    StateBook: TNotebook;
    Diagnosis: TMemo;
    ErrorLog: TMemo;
    Label8: TLabel;
    Label11: TLabel;
    CSMapperAlready: TRadioButton;
    CSTobe: TRadioButton;
    VMMapperAlready: TRadioButton;
    VMTobe: TRadioButton;
    vmremove: TCheckBox;
    csremove: TCheckBox;
    BitBtn2: TBitBtn;
    CSVendorText: TMemo;
    Label3: TLabel;
    RealMode: TRadioButton;
    Prot16Mode: TRadioButton;
    DLLMode: TRadioButton;
    CSAutoDetect: TCheckBox;
    procedure ShowHint(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ClientTabsChange(Sender: TObject; NewTab: Integer;
      var AllowChange: Boolean);
    procedure HeaderSized(Sender: TObject; ASection, AWidth: Integer);
    procedure Header1Sized(Sender: TObject; ASection, AWidth: Integer);
    procedure StateTabsChange(Sender: TObject; NewTab: Integer;
      var AllowChange: Boolean);
    procedure BitBtn1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure CSAutoDetectClick(Sender: TObject);
  private
    { Private declarations }
  Protected
    Clients : TList;
    CSInfo : TCSInfoDefault;
    NeedExtraDrivers : Boolean;
    WantVMMapper : Boolean;
    WantCSMapper : Boolean;
    GotVMMapper : Boolean;
    GotCSMapper : Boolean;
    ActiveVMMapper : Boolean;
    ActiveCSMApper : Boolean;
    Express : Boolean;
    FUserType : TCSType;
  public
    { Public declarations }
    HasCS : Boolean;
    Procedure SetUserType;
    Procedure Go(IsExpress : Boolean);
    Procedure AddError(Operation : String; Error : Integer);
    Procedure Diagnose(Info : String);
    Procedure GetCSInfo;
    Procedure GetClientInfo;
    Procedure FreeClientInfo;
    Procedure DisplayClientInfo(ClientInfo : PClientInfoDefault);
    Procedure RemoveBlanks(Strings : TStrings);
    Procedure doExtraDrivers;
  end;

var
  Setup: TSetup;

implementation

uses About, first;

{$R *.DFM}

procedure TSetup.ShowHint(Sender: TObject);
begin
  StatusBar.Caption := Application.Hint;
end;

procedure TSetup.ExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TSetup.FormCreate(Sender: TObject);
begin
  StateTabs.tabs.Assign(StateBook.Pages);
  Statetabs.TabIndex:=0;
  Clients:=TList.Create;
  Application.OnHint := ShowHint;
end;

Procedure TSetup.GetClientInfo;
var
  Client : TClientHandle;
  ClientInfo : PClientInfoDefault;
  Error : Integer;
begin
  If HasCs then begin
    If GetFirstClient(Client,-1,@Error) then begin
      repeat
        New(ClientInfo);
        ClientInfo^.MaxLen:=Sizeof(ClientInfo^);
        ClientInfo^.Attributes:=[];
        If GetClientInfoDefault(Client,ClientInfo^,@Error) then begin
          Clients.Add(ClientInfo);
          ClientTabs.Tabs.AddObject(StrPas(Addr(ClientInfo^.Data[ClientInfo^.NameOffset])),TObject(ClientInfo));
        end;
      Until not GetNextClient(Client,Client,-1,@Error);
      If ClientTabs.Tabs.Count>0 then
        ClientTabs.TabIndex:=0;
    end else
      AddError('checking clients',Error);
  end;
  If Clienttabs.Tabs.Count=0 then
    TabbedNotebook1.Pages.Delete(TabbedNotebook1.Pages.IndexOf('Active Clients'));
end;

Procedure TSetup.FreeClientInfo;
var
  C : Integer;
begin
  for C:=0 to Clients.Count -1 do begin
    Dispose(PClientInfoDefault(Clients[c]));
  end;
  Clients.Free;
end;

procedure TSetup.FormDestroy(Sender: TObject);
begin
  FreeClientInfo;
end;

procedure TSetup.ClientTabsChange(Sender: TObject;
  NewTab: Integer; var AllowChange: Boolean);
begin
  DisplayClientInfo(PClientInfoDefault(ClientTabs.Tabs.Objects[Newtab]));
end;

procedure TSetup.DisplayClientInfo;
begin
  if @ClientInfo=nil then begin
    MTD.Checked:=False;
    MCD.Checked:=False;
    IOD.Checked:=False;
    Insertion.Checked:=False;
    ExInsertion.Checked:=False;
    VendorText.Clear;
    ClientName.Caption:='';
    Revision.Caption:='';
    RevisionDate.Caption:='';
    Target.Caption:='';
  end else begin
    ClientName.Caption:=StrPas(Addr(ClientInfo^.Data[ClientInfo^.NameOffset]));
    VendorText.Lines.SetText(Addr(ClientInfo^.Data[ClientInfo^.VendorStringOffset]));
    Revision.Caption:=BCD2Version(ClientInfo^.Revision);
    Target.Caption:=BCD2Version(ClientInfo^.CSLevel);
    RevisionDate.Caption:=FormatDateTime('c',CSDateToDate(ClientInfo^.RevDate));
{    RevisionDate.Caption:=Format('%g',[CSDateToDate(ClientInfo^.RevDate)]);}
    With ClientInfo^ do begin
      If rcMemoryClientDriver in Attributes then
        MCD.Checked:=True
      else
        MCD.Checked:=False;
      If rcMemoryTechnologyDriver in Attributes then
        MTD.Checked:=True
      else
        MTD.Checked:=False;
      If rcIODriver in Attributes then
        IOD.Checked:=True
      else
        IOD.Checked:=False;
      If rcCardInsertion in Attributes then
        Insertion.Checked:=True
      else
        Insertion.Checked:=False;
      If rcCardInsertionExclusive in Attributes then
        ExInsertion.Checked:=True
      else
        ExInsertion.Checked:=False;
    end;
  end;
end;

procedure TSetup.HeaderSized(Sender: TObject; ASection,
  AWidth: Integer);
begin
  Revision.Left:=ClientName.Left+Header.SectionWidth[0];
  RevisionDate.Left:=Revision.Left+Header.SectionWidth[1];
  Target.Left:=RevisionDate.Left+Header.SectionWidth[2];
end;

procedure TSetup.Diagnose;
begin
  if Express then
    FirstForm.Log(Info)
  else begin
    Diagnosis.Lines.Add('* '+Info);
    Diagnosis.Lines.Add('');
  end;
end;

procedure TSetup.AddError;
var
  Info : String;
begin
  Info:=Format('!! Error %d ''%s'' while %s',[Error,StrPas(GetError(Error)),Operation]);
  if Express then
    FirstForm.Log(Info)
  else begin
    ErrorLog.Lines.Add(Info);
    ErrorLog.Lines.Add('');
  end;
end;

procedure TSetup.GetCSInfo;
var
  Error : Integer;
  C : TCSType;
  ActiveCSType : TCSType;
  OtherTypes : TCSDetected;
  Versions : longint;
  Win95 : Boolean;
  Ini : TIniFile;
  IsAuto : Boolean;

  function MyGetCardServicesInfo(var CSInfo; Size : Integer; Error : PInteger) : boolean;
  var
    Code : Integer;
  begin
    Code:=CardServ.MapCardServices(ActiveCSType,$0B,0,nil,nil,Size,@CSInfo);
    If Assigned(Error) then begin
      Error^:=Code;
    end;
    MyGetCardServicesInfo:=Code=0;
  end;

  function MyGetCardServicesInfoDefault({ Expects fixed size buffer }
         var CSInfo : TCSInfoDefault; Error : PInteger) : boolean;
  begin
    Result:=MyGetCardServicesInfo(CSInfo,SizeOf(CSInfo),Error);
  end;

begin
  Ini:=TInifile.Create(IniFile);
  try
    ActiveCSType:=TCSType(Ini.ReadInteger(Options,CardServicesType,Integer(csUnknown)));
    IsAuto:=ActiveCSType in [csAuto,csUnknown,csNone];
  finally
    Ini.Free;
  end;

  FUserType:=csUnknown; { this will get filled in by the event handlers when check boxes are set }
  GotCSMapper:=ExtraDrivers.CheckGotCSMapper;
  GotVMMapper:=ExtraDrivers.CheckGotVMMapper;
  Versions:=GetVersion;
  WindowsVersion.Caption:=Format('%d.%d',[LoByte(LoWord(Versions)),HiByte(LoWord(Versions))]);
  DOSVersion.Caption:=Format('%d.%d',[HiByte(HiWord(Versions)),LoByte(HiWord(Versions))]);
  Win95:=LoWord(Versions)=($5F03);
  IsWindows95.Visible:=Win95;
  ActiveCSType:=CheckInstallation(csAuto);
  FillChar(CSInfo,SizeOf(CSInfo),0);
  If (ActiveCSType<>csNone) then begin
    hasCS:=true;
    MyGetCardServicesInfoDefault(CSInfo,@Error);
    CSVersion.Caption:=BCD2Version(CSInfo.CSLevel);
    CSRevision.Caption:=BCD2Version(CSInfo.Revision);
{    ActiveCSType:=Style;
    OtherTypes:=AllStyles;}
    SearchInstallation(csAuto,OtherTypes);

    CardServices.Caption:=StrPas(CSTypes[ActiveCSType]);
    CSSockets.Caption:=IntToStr(CSInfo.SocketCount);
    CSVendortext.Lines.SetText(CSInfo.VendorString);
    RemoveBlanks(CSVendorText.Lines);

    If (csBadRealProt16 in OtherTypes) then begin
      Diagnose('Possible corruption... during install-time test.  After test, restart windows.');
    end;
    RealMode.Enabled:=(csRealMode in OtherTypes);
    Prot16Mode.Enabled:=(csProt16 in OtherTypes);
    DLLMode.Enabled:=(csPhoenix in OtherTypes);
    RealMode.Checked:=ActiveCSType=csRealMode;
    Prot16Mode.Checked:=ActiveCSType=csProt16;
    DLLMode.Checked:=ActiveCSType=csPhoenix;
    CSAutoDetect.Checked:=IsAuto or Express;

    { Now diagnose }
    If CSInfo.CSLevel<$0210 then Diagnose('Card Services outdated.  2.1 or greater required');
    If CSInfo.SocketCount<=0 then Diagnose('No sockets detected');
    If ActiveCSType=csRealMode then begin
      if not GotVMMapper then
        Diagnose('Real Mode card services has been chosen.  Extra drivers needed. '+
                 'This will be installed in the next stage.')
      else
        Diagnose('Real Mode card services has been chosen.  Extra drivers needed are already loaded.');
      WantVMMapper:=True;
      { If it worked, and we have CSMApper then thats WHY it worked and we need it }
      { ewlse it works without so we don't }
      WantCSMapper:=GotCSMapper;

    end else begin
      Diagnose('Your current setup is sufficent and should function correctly.');
    end;
  end else begin
    AddError('While checking card services',Error);

    If Win95 then begin
      Diagnose('No card services detected, card services 2.1 or greater required. '+
               'Windows 95 has its own real mode card services drivers which can be loaded if you wish.');
      If GotCSMapper or GotVMMapper then
        Diagnose('Some of the required drivers have been detected in your startup files, but do not appear to be active. '+
                 'It may be that you have not restarted your machine since these drivers were installed, '+
                  'or that your configuration is wrong.  Setup will fix this.');
      WantVMMapper:=True;
      WantCSMapper:=True;
    end else begin
      Diagnose('No card services detected, card services 2.1 or greater required');
    end;
  end;

  NeedExtraDrivers:=(not hasCS) or (WantVMMapper and not GotVMMapper) or (WantCSMapper and not GotCSMapper);
  If not NeedExtraDrivers then begin
    TabbedNotebook1.Pages.Delete(TabbedNotebook1.Pages.IndexOf('Extra Drivers'));
  end else begin
    CSMapperInfo.WordWrap:=true;
    VMMapperInfo.WordWrap:=true;
    CSMapperAlready.Checked:=GotCSMapper;
    VMMapperAlready.Checked:=GotVMMapper;
    CSTobe.Checked:=WantCSMapper and not GotCSMapper;
    VMTobe.Checked:=WantVMMapper and not GotVMMapper;
    CSMapperGroup.Visible:=WantCSMapper or GotCSMapper;
    VMMapperGroup.Visible:=WantVMMapper or GotVMMapper;
  end;
end;

procedure TSetup.Header1Sized(Sender: TObject; ASection,
  AWidth: Integer);            
begin
  CSVersion.Left:=CardServices.Left+Header1.SectionWidth[0];
  CSRevision.Left:=CSVersion.Left+Header1.SectionWidth[1];
  CSSockets.Left:=CSRevision.Left+Header1.SectionWidth[1];
end;

procedure TSetup.RemoveBlanks;
var
  C : Integer;
begin
  C:=0; while C<Strings.Count do
    If Strings[c]='' then
      Strings.Delete(c)
    else
      inc(c);
end;

procedure TSetup.StateTabsChange(Sender: TObject; NewTab: Integer;
  var AllowChange: Boolean);
begin
  StateBook.PageIndex:=NewTab;
end;

procedure TSetup.BitBtn1Click(Sender: TObject);
begin
  DoExtraDrivers;
end;

procedure TSetup.DoExtraDrivers;
var
  Dir : String;
  p : pointer;
  Driver : String;
  DCS, DVM : Boolean;
begin
  Try
{    Application.CreateForm(TExtraDrivers,ExtraDrivers);}
    { Now insert drivers themselves }
    If WantCSMapper and ((not GotCSMapper) or (not ActiveCSMapper)) then begin
      Dir[0]:=Char(GetSystemDirectory(@Dir[1],High(Dir)));
      ExtraDrivers.ConfigDrivers.Items.Clear;
      Driver:=Format('%s\csmapper.sys',[Dir]);
      If not FileExists(Driver) then begin
        MessageDlg(Format('The driver ''%s'' could not be found.  It is normally installed '+
              'by windows95 when your PCCard slot is detected.  This file is required.',[driver]),
              mtError,[mbOK,mbHelp],20);
      end;
      ExtraDrivers.ConfigDrivers.Items.AddObject(format('device=%s',[Driver]),
        pointer(longint(ExtraDrivers.CSMapperInFile+1)));
      DCS:=True;
    end else
      DCS:=False;

    If WantVMMapper and ((not GotVMMapper) or (not ActiveVMMapper)) then begin
      Dir:=ExtractFilePath(ParamStr(0));
      ExtraDrivers.AutoexecDrivers.Items.Clear;
      Driver:=format('%svmmapper.exe',[Dir]);
      If not FileExists(Driver) then begin
    {        CheckLocation(Driver)};
        MessageDlg(Format('The driver ''%s'' could not be found.  It is normally installed '+
              'with SMS.  This file is required for proper virtual machine support.',[driver]),
              mtError,[mbOK,mbHelp],30);
      end;
      ExtraDrivers.AutoexecDrivers.Items.AddObject(Driver,
        pointer(longint(ExtraDrivers.VMMapperInFile+1)));
      DVM:=True;
    end else
      DVM:=False;


    If ExtraDrivers.Go(Express,DCS,DVM) then { closed on good terms }
      SetType(FUserType);
    Close;
  finally
{    ExtraDrivers.Free;}
  end;
end;

procedure TSetup.FormShow(Sender: TObject);
const
  FirstShow : Boolean=True;
begin
  if not FirstShow then exit;
  GetCSInfo;
  GetClientinfo;
  FirstShow:=True;
end;

procedure TSetup.BitBtn2Click(Sender: TObject);
begin
  if NeedExtraDrivers then
    TabbedNoteBook1.PageIndex:=TabbedNoteBook1.PageIndex+1
  else begin
    { Closing on good terms, so.... }
    SetType(FUserType);
    close;
  end;
end;

procedure TSetup.Go;
begin
  Express:=IsExpress;
  If not Express then ShowModal else begin
    GetCSInfo;
    If NeedExtraDrivers then
      DoExtraDrivers;
    SetType(FUserType);
  end;
end;

procedure TSetup.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  If Key=VK_F1 then begin
    With TabbedNoteBook1 do
      Application.HelpContext((Pages.Objects[PageIndex] as TWinControl).HelpContext);
  end;
end;

procedure TSetup.CSAutoDetectClick(Sender: TObject);
begin
  If CSAutoDetect.Checked then
    FUserType:=csAuto
  else
    SetUserType;
end;

procedure TSetup.SetUserType;
begin
  If RealMode.Checked then
    FUserType:=csRealMode
  else if Prot16Mode.Checked then
    FUserType:=csProt16
  else if DLLMode.Checked then
    FUserType:=csPhoenix
  else
    FUserType:=csAuto; { stick to auto... }
end;

end.

