unit Clients;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Menus,
  Dialogs, StdCtrls, Buttons, ExtCtrls, cardserv, Tabs, csTypes2, sysutils,
  utils, TabNotBk, Inifiles, dllini, messages, explbtn, W95Meter, Gauge95;

type
  TBuffer = Array[0..8192] of char;
  PBuffer=^TBuffer;

  TSetup = class(TForm)
    StatusBar: TPanel;
    TabbedNotebook1: TTabbedNotebook;
    GroupBox1: TGroupBox;
    ClientName: TLabel;
    Revision: TLabel;
    RevisionDate: TLabel;
    Target: TLabel;
    Header: THeader;
    ClientTabs: TTabSet;
    Header1: THeader;
    CardServices: TLabel;
    CSVersion: TLabel;
    CSRevision: TLabel;
    CSSockets: TLabel;
    Panel1: TPanel;
    IOD: TRadioButton;
    MTD: TRadioButton;
    MCD: TRadioButton;
    Insertion: TCheckBox;
    ExInsertion: TCheckBox;
    VendorText: TMemo;
    Label2: TLabel;
    Label1: TLabel;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    Label6: TLabel;
    Label7: TLabel;
    WindowsVersion: TLabel;
    DOSVersion: TLabel;
    IsWindows95: TLabel;
    Label9: TLabel;
    StateTabs: TTabSet;
    StateBook: TNotebook;
    Diagnosis: TMemo;
    Label8: TLabel;
    BitBtn2: TBitBtn;
    CSVendorText: TMemo;
    Label3: TLabel;
    GroupBox5: TGroupBox;
    SaveButton: TExplorerButton;
    LoadButton: TExplorerButton;
    GroupBox4: TGroupBox;
    CardSize: TComboBox;
    GroupBox6: TGroupBox;
    S0: TRadioButton;
    S4: TRadioButton;
    S1: TRadioButton;
    S5: TRadioButton;
    S2: TRadioButton;
    S6: TRadioButton;
    S3: TRadioButton;
    S7: TRadioButton;
    SaveDialog: TSaveDialog;
    OpenDialog: TOpenDialog;
    Gauge: TGauge95;
    Panel2: TPanel;
    RealMode: TRadioButton;
    Prot16Mode: TRadioButton;
    DLLMode: TRadioButton;
    CSAutoDetect: TCheckBox;
    procedure ShowHint(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ClientTabsChange(Sender: TObject; NewTab: Integer;
      var AllowChange: Boolean);
    procedure HeaderSized(Sender: TObject; ASection, AWidth: Integer);
    procedure Header1Sized(Sender: TObject; ASection, AWidth: Integer);
    procedure StateTabsChange(Sender: TObject; NewTab: Integer;
      var AllowChange: Boolean);
    procedure FormShow(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure CSAutoDetectClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure LoadButtonClick(Sender: TObject);
    procedure S0Click(Sender: TObject);
  private
    { Private declarations }
  Protected
    Clients : TList;
    CSInfo : TCSInfoDefault;
    FUserType : TCSType;
    CS_Handle : TClientHandle;
    Registered : Boolean;
    Sockets : Array[0..7] of TRadioButton;
  public
    { Public declarations }
    HasCS : Boolean;
    Procedure CM_Message(Var MSG : TMessage); message WM_Command; { To detect cardservices callbacks }
    Procedure ProcessCallback(var MSG : TMessage; var handled : boolean); { To process card services callbacks }
    Procedure SetUserType;
    Procedure AddError(Operation : String; Error : Integer);
    Procedure Diagnose(Info : String);
    Procedure GetCSInfo;
    Procedure GetClientInfo;
    Procedure FreeClientInfo;
    Procedure DisplayClientInfo(ClientInfo : PClientInfoDefault);
    Procedure RemoveBlanks(Strings : TStrings);
    Procedure Register;
    Procedure DeRegister;
    Procedure CardEvent(Event : Integer; Socket : TSocket);
    Function GetSize : Longint;
    Procedure SetSize(Size : longint);
    Function Calcsize(Socket : TSocket) : longint;
    Function getSocket : TSocket;
    Procedure SaveToFile(Socket : TSocket; Size : longint; Filename : String);
    Procedure LoadFromFile(Socket : TSocket; Filename : String);
  end;

var
  Setup: TSetup;

implementation

uses About;

{$R *.DFM}

Function Min(a, b : longint) : longint;
begin
  If a<b then Result:=a else result:=b;
end;

procedure TSetup.ShowHint(Sender: TObject);
begin
  StatusBar.Caption := Application.Hint;
end;

procedure TSetup.ExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TSetup.FormCreate(Sender: TObject);
begin
  StateTabs.tabs.Assign(StateBook.Pages);
  Statetabs.TabIndex:=0;
  Clients:=TList.Create;
  Application.OnHint := ShowHint;
  Sockets[0]:=S0;
  Sockets[1]:=S1;
  Sockets[2]:=S2;
  Sockets[3]:=S3;
  Sockets[4]:=S4;
  Sockets[5]:=S5;
  Sockets[6]:=S6;
  Sockets[7]:=S7;
end;

Procedure TSetup.GetClientInfo;
var
  Client : TClientHandle;
  ClientInfo : PClientInfoDefault;
  Error : Integer;
begin
  If HasCs then begin
    If GetFirstClient(Client,-1,@Error) then begin
      repeat
        New(ClientInfo);
        ClientInfo^.MaxLen:=Sizeof(ClientInfo^);
        ClientInfo^.Attributes:=[];
        If GetClientInfoDefault(Client,ClientInfo^,@Error) then begin
          Clients.Add(ClientInfo);
          ClientTabs.Tabs.AddObject(StrPas(Addr(ClientInfo^.Data[ClientInfo^.NameOffset])),TObject(ClientInfo));
        end;
      Until not GetNextClient(Client,Client,-1,@Error);
      If ClientTabs.Tabs.Count>0 then
        ClientTabs.TabIndex:=0;
    end else
      AddError('checking clients (csmapper.sys does not support this)',Error);
  end;
  If Clienttabs.Tabs.Count=0 then
    TabbedNotebook1.Pages.Delete(TabbedNotebook1.Pages.IndexOf('Active Clients'));
end;

Procedure TSetup.FreeClientInfo;
var
  C : Integer;
begin
  for C:=0 to Clients.Count -1 do begin
    Dispose(PClientInfoDefault(Clients[c]));
  end;
  Clients.Free;
end;

procedure TSetup.FormDestroy(Sender: TObject);
begin
  FreeClientInfo;
end;

procedure TSetup.ClientTabsChange(Sender: TObject;
  NewTab: Integer; var AllowChange: Boolean);
begin
  DisplayClientInfo(PClientInfoDefault(ClientTabs.Tabs.Objects[Newtab]));
end;

procedure TSetup.DisplayClientInfo;
begin
  if @ClientInfo=nil then begin
    MTD.Checked:=False;
    MCD.Checked:=False;
    IOD.Checked:=False;
    Insertion.Checked:=False;
    ExInsertion.Checked:=False;
    VendorText.Clear;
    ClientName.Caption:='';
    Revision.Caption:='';
    RevisionDate.Caption:='';
    Target.Caption:='';
  end else begin
    ClientName.Caption:=StrPas(Addr(ClientInfo^.Data[ClientInfo^.NameOffset]));
    VendorText.Lines.SetText(Addr(ClientInfo^.Data[ClientInfo^.VendorStringOffset]));
    Revision.Caption:=BCD2Version(ClientInfo^.Revision);
    Target.Caption:=BCD2Version(ClientInfo^.CSLevel);
    RevisionDate.Caption:=FormatDateTime('c',CSDateToDate(ClientInfo^.RevDate));
{    RevisionDate.Caption:=Format('%g',[CSDateToDate(ClientInfo^.RevDate)]);}
    With ClientInfo^ do begin
      If rcMemoryClientDriver in Attributes then
        MCD.Checked:=True
      else
        MCD.Checked:=False;
      If rcMemoryTechnologyDriver in Attributes then
        MTD.Checked:=True
      else
        MTD.Checked:=False;
      If rcIODriver in Attributes then
        IOD.Checked:=True
      else
        IOD.Checked:=False;
      If rcCardInsertion in Attributes then
        Insertion.Checked:=True
      else
        Insertion.Checked:=False;
      If rcCardInsertionExclusive in Attributes then
        ExInsertion.Checked:=True
      else
        ExInsertion.Checked:=False;
    end;
  end;
end;

procedure TSetup.HeaderSized(Sender: TObject; ASection,
  AWidth: Integer);
begin
  Revision.Left:=ClientName.Left+Header.SectionWidth[0];
  RevisionDate.Left:=Revision.Left+Header.SectionWidth[1];
  Target.Left:=RevisionDate.Left+Header.SectionWidth[2];
end;

procedure TSetup.Diagnose;
begin
  Diagnosis.Lines.Add('* '+Info);
  Diagnosis.Lines.Add('');
end;

procedure TSetup.AddError;
var
  Info : String;
begin
  Info:=Format('!! Error %d ''%s'' while %s',[Error,StrPas(GetError(Error)),Operation]);
  Diagnosis.Lines.Add(Info);
  Diagnosis.Lines.Add('');
end;

procedure TSetup.GetCSInfo;
var
  Error : Integer;
  C : TCSType;
  ActiveCSType : TCSType;
  OtherTypes : TCSDetected;
  Versions : longint;
  Win95 : Boolean;
  Ini : TIniFile;
  IsAuto : Boolean;

  function MyGetCardServicesInfo(var CSInfo; Size : Integer; Error : PInteger) : boolean;
  var
    Code : Integer;
  begin
    Code:=CardServ.MapCardServices(ActiveCSType,$0B,0,nil,nil,Size,@CSInfo);
    If Assigned(Error) then begin
      Error^:=Code;
    end;
    MyGetCardServicesInfo:=Code=0;
  end;

  function MyGetCardServicesInfoDefault({ Expects fixed size buffer }
         var CSInfo : TCSInfoDefault; Error : PInteger) : boolean;
  begin
    Result:=MyGetCardServicesInfo(CSInfo,SizeOf(CSInfo),Error);
  end;

begin
  Ini:=TInifile.Create(IniFile);
  try
    ActiveCSType:=TCSType(Ini.ReadInteger(Options,CardServicesType,Integer(csUnknown)));
    IsAuto:=ActiveCSType in [csAuto,csUnknown,csNone];
  finally
    Ini.Free;
  end;

  FUserType:=csUnknown; { this will get filled in by the event handlers when check boxes are set }
  Versions:=GetVersion;
  WindowsVersion.Caption:=Format('%d.%d',[LoByte(LoWord(Versions)),HiByte(LoWord(Versions))]);
  DOSVersion.Caption:=Format('%d.%d',[HiByte(HiWord(Versions)),LoByte(HiWord(Versions))]);
  Win95:=LoWord(Versions)=($5F03);
  IsWindows95.Visible:=Win95;
  ActiveCSType:=CheckInstallation(csAuto);
  FillChar(CSInfo,SizeOf(CSInfo),0);
  If (ActiveCSType<>csNone) then begin
    hasCS:=true;
    MyGetCardServicesInfoDefault(CSInfo,@Error);
    CSVersion.Caption:=BCD2Version(CSInfo.CSLevel);
    CSRevision.Caption:=BCD2Version(CSInfo.Revision);
{    ActiveCSType:=Style;
    OtherTypes:=AllStyles;}
    SearchInstallation(csAuto,OtherTypes);

    CardServices.Caption:=StrPas(CSTypes[ActiveCSType]);
    CSSockets.Caption:=IntToStr(CSInfo.SocketCount);
    CSVendortext.Lines.SetText(CSInfo.VendorString);
    RemoveBlanks(CSVendorText.Lines);

    If (csBadRealProt16 in OtherTypes) then begin
      Diagnose('Possible corruption... during install-time test.  After test, restart windows.');
    end;
    RealMode.Enabled:=(csRealMode in OtherTypes);
    Prot16Mode.Enabled:=(csProt16 in OtherTypes);
    DLLMode.Enabled:=(csPhoenix in OtherTypes);
    RealMode.Checked:=ActiveCSType=csRealMode;
    Prot16Mode.Checked:=ActiveCSType=csProt16;
    DLLMode.Checked:=ActiveCSType=csPhoenix;
    CSAutoDetect.Checked:=IsAuto;

    { Now diagnose }
    If CSInfo.CSLevel<$0210 then Diagnose('Card Services outdated.  2.1 or greater required');
    If CSInfo.SocketCount<=0 then Diagnose('No sockets detected');
    If ActiveCSType=csRealMode then begin
      Diagnose('Real Mode card services has been detected');
    end else begin
      Diagnose('No card services detected, card services 2.1 or greater required');
    end;
  end;
end;

procedure TSetup.Header1Sized(Sender: TObject; ASection,
  AWidth: Integer);
begin
  CSVersion.Left:=CardServices.Left+Header1.SectionWidth[0];
  CSRevision.Left:=CSVersion.Left+Header1.SectionWidth[1];
  CSSockets.Left:=CSRevision.Left+Header1.SectionWidth[1];
end;

procedure TSetup.RemoveBlanks;
var
  C : Integer;
begin
  C:=0; while C<Strings.Count do
    If Strings[c]='' then
      Strings.Delete(c)
    else
      inc(c);
end;

procedure TSetup.StateTabsChange(Sender: TObject; NewTab: Integer;
  var AllowChange: Boolean);
begin
  StateBook.PageIndex:=NewTab;
end;

procedure TSetup.FormShow(Sender: TObject);
const
  FirstShow : Boolean=True;
var
  C : Integer;
begin
  if not FirstShow then exit;
  FirstShow:=True;
  GetCSInfo;
  Register;
  GetClientinfo;
  For c:=SocketCount to 7 do
    Sockets[c].Visible:=False;
end;

procedure TSetup.BitBtn2Click(Sender: TObject);
begin
  close;
end;

procedure TSetup.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  If Key=VK_F1 then begin
    With TabbedNoteBook1 do
      Application.HelpContext((Pages.Objects[PageIndex] as TWinControl).HelpContext);
  end;
end;

procedure TSetup.CSAutoDetectClick(Sender: TObject);
begin
  If CSAutoDetect.Checked then
    FUserType:=csAuto
  else
    SetUserType;
end;

procedure TSetup.SetUserType;
begin
  If RealMode.Checked then
    FUserType:=csRealMode
  else if Prot16Mode.Checked then
    FUserType:=csProt16
  else if DLLMode.Checked then
    FUserType:=csPhoenix
  else
    FUserType:=csAuto; { stick to auto... }
end;

Procedure TSetup.Register;
var
  Error : Integer;
begin
  If RegisterClient(CS_Handle,[rcMemoryClientDriver,rcCardInsertion],
      AllEvents,$210,'TestClient',
      'Campbell Scientific Ltd'+#13+'(C) 1997 Campbell Scientific Ltd',
      $0220,EncodeCSDate(1997,2,18),
      @DefaultCallback,Self.Handle,0,0,@Error) then
    Registered:=True
  else
    Registered:=False;
end;

Procedure TSetup.DeRegister;
var
  Error : Integer;
begin
  If not DeRegisterClient(CS_Handle,@Error) then
    MessageDlg('Error de-registereing',mtError,[mbOK],0);
  Registered:=False;
end;

procedure TSetup.FormHide(Sender: TObject);
begin
  if Registered then Deregister;
end;

Procedure TSetup.CM_Message;
var
  Handled : boolean;
begin
  Handled:=false;
  ProcessCallback(msg, Handled);
  If not Handled then inherited;
end;

Procedure TSetup.ProcessCallback;
begin
  { event is HiWord(LParam) and socket is LoWord(LParam) }
  If (Msg.Msg=WM_Command) and (msg.wParam=IDM_Callback) then begin
    CardEvent(HiWord(msg.lParam),LoWord(msg.lParam));
  end;
end;

Procedure TSetup.CardEvent;
begin
  Case Event of
    cbCardInsertion: begin
                       Sockets[Socket].Enabled:=True;
                       If GetSocket=-1 then
                         Sockets[Socket].Checked:=true;
                     end;
    cbCardRemoval:   With Sockets[Socket] do begin
                       Enabled:=False;
                       Checked:=False;
                       S0Click(nil);
                     end;
  end;

end;

procedure TSetup.SaveButtonClick(Sender: TObject);
var
  Size : longint;
  Socket : TSocket;
begin
  Size:=GetSize;
  Socket:=GetSocket;
  if SaveDialog.Execute then begin
    SaveToFile(Socket,Size,SaveDialog.Filename);
  end;
end;

procedure TSetup.LoadButtonClick(Sender: TObject);
var
  Socket : TSocket;
begin
  Socket:=GetSocket;
  if OpenDialog.Execute then begin
    LoadFromFile(Socket,OpenDialog.Filename);
  end;
end;

Function TSetup.getSize;
var
  text : String;
  value : longint;
  Error : Integer;

  Function GetMultiplier(C : Char) : longint;
  begin
    Case C of
      'M','m': Result:=1024*1024;
      'K','k': Result:=1024;
    else
      Raise Exception.CreateFmt('Unknown Multiplier: %c',[C]);
    end;
  end;

  Function GetVal(Text : string) : longint;
  begin
    Val(Text,Value,Error);
    If Error<>0 then
     If Error=length(text) then begin
       Value:=getVal(Copy(Text,1,Error-1))*getMultiplier(Text[Error])
     end else
       raise Exception.CreateFmt('Improper size repesentation ending with: %s',[Copy(Text,Error,255)]);
    Result:=Value;
  end;

begin
  text:=CardSize.Text;
  Result:=GetVal(Text);
end;

Procedure TSetup.SetSize;
var
  index : Integer;
  text : String;
begin
  { Factorise out size into K or M }
  if Size=0 then
    CardSize.ItemIndex:=0
  else begin
    If (Size>=1024*1024) and ((Size Mod (1024*1024))=0) then
      text:=IntToStr(Size div (1024*1024))+'M'
    else if (Size>=1024) and ((Size mod 1024)=0) then
      text:=IntToStr(Size div 1024)+'K'
    else
      Text:=IntToStr(Size);
    Index:=CardSize.Items.IndexOf(Text);
    If Index>=0 then
      CardSize.ItemIndex:=Index
    else
      CardSize.text:=Text;
  end;
end;

Function TSetup.Calcsize;
var
  LastAddress, Address : longint;

  Function Test(Socket : TSocket; Address : longint) : boolean;
  var
    Error : Integer;
    buffer : byte;
  begin
    CopyFromCard(CS_Handle,Socket,Buffer,Address,Sizeof(Buffer),@Error);
    Result:=Error=0;
  end;

begin
  { sacn though reading bits of card till it fails (up to max of 32M, then say don't know }
  { we read a byte in the next K boundry, if we fail we think we have it }
  LastAddress:=0;
  Address:=1024; { start with 1K}
  repeat
    If not Test(Socket,Address) then break;
    LastAddress:=Address;
    Address:=Address+Address;

    If Address>=32*1024*1024 then begin
      Address:=0;
      break;
    end;
  until false;
  If Address<>0 then begin
    { LastAddress is the last boundry that worked, and Address is the one that failed }
    { Quick test of Address-1 for success, if not halve the interval }
    If not Test(Socket,Address-1) then begin{ halve interval }
      { can't be bothered }
      Address:=0;
    end;
  end;
  Result:=Address;
end;

Function TSetup.GetSocket;
var
  C : Integer;
begin
  Result:=-1;
  For C:=0 to SocketCount-1 do begin
    if Sockets[C].Checked then begin
      Result:=C;
      break;
    end;
  end;
end;

Procedure TSetup.SaveToFile;
var
  BlockSize : Longint;
  Stream : TFileStream;
  Buffer : PBuffer;
  address : longint;
  Error : Integer;
begin
  Stream:=TFileStream.Create(Filename,fmCreate);
  New(Buffer);
  Gauge.MaxValue:=Size;
  Gauge.Progress:=0;
  Gauge.Visible:=true;
  try
    address:=0;
    while address<Size do begin
      BlockSize:=Min(Sizeof(Buffer^),Size-Address);
      CopyfromCard(CS_Handle,Socket,Buffer^,address,BlockSize,@Error);
      if Error<>0 then
        Raise ECSException.CreateMessage(Error,Format('While reading %d bytes from %d',[BlockSize,Address]));
      If BlockSize<>Stream.Write(Buffer^,Blocksize) then
        Raise Exception.CreateFmt('Error while writing %d bytes to file at position %d',[BlockSize,Stream.Position]);
      inc(address,BlockSize);
      Gauge.Progress:=Gauge.Progress+BlockSize;
    end;
  finally
    Dispose(Buffer);
    Stream.Free;
    Gauge.Visible:=False;
  end;
end;

Procedure TSetup.LoadFromFile;
var
  BlockSize : Longint;
  Stream : TFileStream;
  Buffer : PBuffer;
  address : longint;
  Error : Integer;
begin
  Stream:=TFileStream.Create(Filename,fmOpenRead);
  New(Buffer);
  Gauge.MaxValue:=Stream.Size;
  Gauge.Progress:=0;
  Gauge.Visible:=true;
  try
    address:=0;
    repeat
      BlockSize:=Stream.Read(Buffer^,SizeOf(Buffer^));
      If BlockSize=0 then break;
      CopyToCard(CS_Handle,Socket,Buffer^,address,BlockSize,True,@Error);
      if Error<>0 then
        Raise ECSException.CreateMessage(Error,Format('While writing %d bytes to %d',[BlockSize,Address]));
      inc(address,BlockSize);
      Gauge.Progress:=Gauge.Progress+BlockSize;
    until BlockSize=0;
  finally
    Dispose(Buffer);
    Stream.Free;
    Gauge.Visible:=False;
  end;
end;


procedure TSetup.S0Click(Sender: TObject);
var
  Socket : TSocket;
begin
  Socket:=getSocket;
  If Socket>=0 then
    SetSize(CalcSize(Socket))
  else
    SetSize(0);
end;

end.

